import { relations } from "drizzle-orm";
import {
  boolean,
  integer,
  pgTable,
  serial,
  smallint,
  text,
  timestamp,
  varchar,
} from "drizzle-orm/pg-core";

export const users = pgTable("user", {
  id: varchar("id", { length: 24 }).primaryKey(),
  discord_id: text("discord_id").notNull(),
  name: text("name"),
  image: text("image"),
});

export const sessions = pgTable("session", {
  id: varchar("id", { length: 24 }).primaryKey(),
  userId: varchar("user_id", { length: 24 })
    .notNull()
    .references(() => users.id, { onDelete: "cascade" }),
  expiresAt: timestamp("expires_at", {
    withTimezone: true,
    mode: "date",
  }).notNull(),
});

export const discordTokens = pgTable("tokens", {
  userId: varchar("user_id", { length: 24 })
    .primaryKey()
    .references(() => users.id, { onDelete: "cascade" }),
  refreshToken: text("refresh_token").notNull(),
  accessToken: text("access_token").notNull(),
  expiresAt: timestamp("expires_at", { mode: "date" }).notNull(),
});

export const matchPlannings = pgTable("match_planning", {
  id: serial("id").primaryKey(),
  channelId: varchar("channel_id", { length: 20 }).notNull(),
  matchtype: varchar("match_type", { length: 50 }).notNull(),
  createrId: varchar("creater_id", { length: 20 }).notNull(),
  roleId: varchar("role_id", { length: 20 }).notNull(),
  opponentName: varchar("opponent_name", { length: 100 }).notNull(),
  messageId: varchar("message_id", { length: 20 }).notNull(),
  utc_ts: timestamp("utc_ts").notNull(),
  guildId: varchar("guild_id", { length: 20 })
    .notNull()
    .references(() => guilds.id, { onDelete: "cascade" }),
});

export const matchPlanningsRelations = relations(matchPlannings, ({ one }) => ({
  guild: one(guilds, {
    fields: [matchPlannings.guildId],
    references: [guilds.id],
  }),
}));

export const guilds = pgTable("guild", {
  id: varchar("id", { length: 20 }).primaryKey(),
  timezone: text("timezone").notNull(),
});

export const guildsRelations = relations(guilds, ({ one, many }) => ({
  matches: many(matchPlannings),
  timePlanning: one(timePlannings, {
    fields: [guilds.id],
    references: [timePlannings.guildId],
  }),
}));

export const timePlannings = pgTable("time_planning", {
  id: serial("id").primaryKey(),
  guildId: varchar("guild_id", { length: 20 })
    .notNull()
    .unique()
    .references(() => guilds.id, {
      onDelete: "cascade",
    }),
  channelId: varchar("channel_id", { length: 20 }).notNull(),
  target_interval: smallint("target_interval").notNull(),
  roles: boolean("roles").notNull(),
  isAvailableRoleId: varchar("is_available_role_id", { length: 20 }),
  wantsToBeNotifieRoledId: varchar("wants_to_be_notified_role_id", {
    length: 20,
  }),
});

export const timePlanningsRelations = relations(
  timePlannings,
  ({ one, many }) => ({
    guild: one(guilds, {
      fields: [timePlannings.guildId],
      references: [guilds.id],
    }),
    messages: many(tpMessages),
  }),
);

export const tpMessages = pgTable("tp_message", {
  messageId: varchar("message_id", { length: 20 }).primaryKey(),
  day: smallint("day").notNull(),
  planId: integer("plan_id")
    .notNull()
    .references(() => timePlannings.id, { onDelete: "cascade" }),
});

export const tpMessagesRelations = relations(tpMessages, ({ one }) => ({
  plan: one(timePlannings, {
    fields: [tpMessages.planId],
    references: [timePlannings.id],
  }),
}));
